@@ -17,40 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-
-#include "atk.h"
+#include "atktext.h"
 #include "atkmarshal.h"
+#include "atk-enum-types.h"
 
 #include <string.h>
 
-/**
- * SECTION:atktext
- * @Short_description: The ATK interface implemented by components
- *  with text content.
- * @Title:AtkText
- *
- * #AtkText should be implemented by #AtkObjects on behalf of widgets
- * that have text content which is either attributed or otherwise
- * non-trivial.  #AtkObjects whose text content is simple,
- * unattributed, and very brief may expose that content via
- * #atk_object_get_name instead; however if the text is editable,
- * multi-line, typically longer than three or four words, attributed,
- * selectable, or if the object already uses the 'name' ATK property
- * for other information, the #AtkText interface should be used to
- * expose the text content.  In the case of editable text content,
- * #AtkEditableText (a subtype of the #AtkText interface) should be
- * implemented instead.
- *
- *  #AtkText provides not only traversal facilities and change
- * notification for text content, but also caret tracking and glyph
- * bounding box calculations.  Note that the text strings are exposed
- * as UTF-8, and are therefore potentially multi-byte, and
- * caret-to-byte offset mapping makes no assumptions about the
- * character length; also bounding box glyph-to-offset mapping may be
- * complex for languages which use ligatures.
- */
-
 static GPtrArray *extra_attributes = NULL;
 
 enum {
@@ -60,6 +32,7 @@ enum {
   TEXT_ATTRIBUTES_CHANGED,
   TEXT_INSERT,
   TEXT_REMOVE,
+  TEXT_UPDATE,
   LAST_SIGNAL
 };
 
@@ -135,14 +108,6 @@ static const guint8 underline_offsets[] = {
   0, 5, 12, 19, 23
 };
 
-static const char text_position[] =
-  "baseline\0"
-  "super\0"
-  "sub\0";
-static const guint8 text_position_offsets[] = {
-  0, 9, 15,
-};
-
 static void atk_text_base_init (AtkTextIface *class);
 
 static void atk_text_real_get_range_extents  (AtkText          *text,
@@ -197,21 +162,6 @@ atk_text_base_init (AtkTextIface *class)
       class->get_range_extents = atk_text_real_get_range_extents; 
       class->get_bounded_ranges = atk_text_real_get_bounded_ranges; 
 
-      /**
-       * AtkText::text-changed:
-       * @atktext: the object which received the signal.
-       * @arg1: The position (character offset) of the insertion or deletion.
-       * @arg2: The length (in characters) of text inserted or deleted.
-       *
-       * The "text-changed" signal is emitted when the text of the
-       * object which implements the AtkText interface changes, This
-       * signal will have a detail which is either "insert" or
-       * "delete" which identifies whether the text change was an
-       * insertion or a deletion.
-       *
-       * Deprecated: 2.9.4: Use #AtkObject::text-insert or
-       * #AtkObject::text-remove instead.
-       */
       atk_text_signals[TEXT_CHANGED] =
 	g_signal_new ("text_changed",
 		      ATK_TYPE_TEXT,
@@ -222,18 +172,6 @@ atk_text_base_init (AtkTextIface *class)
 		      G_TYPE_NONE,
 		      2, G_TYPE_INT, G_TYPE_INT);
 
-      /**
-       * AtkText::text-insert:
-       * @atktext: the object which received the signal.
-       * @arg1: The position (character offset) of the insertion.
-       * @arg2: The length (in characters) of text inserted.
-       * @arg3: The new text inserted
-       *
-       * The "text-insert" signal is emitted when a new text is
-       * inserted. If the signal was not triggered by the user
-       * (e.g. typing or pasting text), the "system" detail should be
-       * included.
-       */
       atk_text_signals[TEXT_INSERT] =
 	g_signal_new ("text_insert",
 		      ATK_TYPE_TEXT,
@@ -244,18 +182,6 @@ atk_text_base_init (AtkTextIface *class)
 		      G_TYPE_NONE,
 		      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING);
 
-      /**
-       * AtkText::text-remove:
-       * @atktext: the object which received the signal.
-       * @arg1: The position (character offset) of the removal.
-       * @arg2: The length (in characters) of text removed.
-       * @arg3: The old text removed
-       *
-       * The "text-remove" signal is emitted when a new text is
-       * removed. If the signal was not triggered by the user
-       * (e.g. typing or pasting text), the "system" detail should be
-       * included.
-       */
       atk_text_signals[TEXT_REMOVE] =
 	g_signal_new ("text_remove",
 		      ATK_TYPE_TEXT,
@@ -266,15 +192,16 @@ atk_text_base_init (AtkTextIface *class)
 		      G_TYPE_NONE,
 		      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING);
 
-      /**
-       * AtkText::text-caret-moved:
-       * @atktext: the object which received the signal.
-       * @arg1: The new position of the text caret.
-       *
-       * The "text-caret-moved" signal is emitted when the caret
-       * position of the text of an object which implements AtkText
-       * changes.
-       */
+      atk_text_signals[TEXT_UPDATE] =
+	g_signal_new ("text_update",
+		      ATK_TYPE_TEXT,
+		      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+		      0,
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT_INT_STRING,
+		      G_TYPE_NONE,
+		      4, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING);
+
       atk_text_signals[TEXT_CARET_MOVED] =
 	g_signal_new ("text_caret_moved",
 		      ATK_TYPE_TEXT,
@@ -284,14 +211,6 @@ atk_text_base_init (AtkTextIface *class)
 		      g_cclosure_marshal_VOID__INT,
 		      G_TYPE_NONE,
 		      1, G_TYPE_INT);
-
-      /**
-       * AtkText::text-selection-changed:
-       * @atktext: the object which received the signal.
-       *
-       * The "text-selection-changed" signal is emitted when the
-       * selected text of an object which implements AtkText changes.
-       */
       atk_text_signals[TEXT_SELECTION_CHANGED] =
         g_signal_new ("text_selection_changed",
                       ATK_TYPE_TEXT,
@@ -300,14 +219,6 @@ atk_text_base_init (AtkTextIface *class)
                       (GSignalAccumulator) NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
-      /**
-       * AtkText::text-attributes-changed:
-       * @atktext: the object which received the signal.
-       *
-       * The "text-attributes-changed" signal is emitted when the text
-       * attributes of the text of an object which implements AtkText
-       * changes.
-       */
       atk_text_signals[TEXT_ATTRIBUTES_CHANGED] =
         g_signal_new ("text_attributes_changed",
                       ATK_TYPE_TEXT,
@@ -325,14 +236,13 @@ atk_text_base_init (AtkTextIface *class)
 /**
  * atk_text_get_text:
  * @text: an #AtkText
- * @start_offset: a starting character offset within @text
- * @end_offset: an ending character offset within @text, or -1 for the end of the string.
+ * @start_offset: start position
+ * @end_offset: end position, or -1 for the end of the string.
  *
  * Gets the specified text.
  *
  * Returns: a newly allocated string containing the text from @start_offset up
- *          to, but not including @end_offset. Use g_free() to free the returned
- *          string.
+ *   to, but not including @end_offset. Use g_free() to free the returned string.
  **/
 gchar*
 atk_text_get_text (AtkText      *text,
@@ -358,11 +268,11 @@ atk_text_get_text (AtkText      *text,
 /**
  * atk_text_get_character_at_offset:
  * @text: an #AtkText
- * @offset: a character offset within @text
+ * @offset: position
  *
  * Gets the specified text.
  *
- * Returns: the character at @offset or 0 in the case of failure.
+ * Returns: the character at @offset.
  **/
 gunichar
 atk_text_get_character_at_offset (AtkText      *text,
@@ -385,17 +295,50 @@ atk_text_get_character_at_offset (AtkText      *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: (out): the starting character offset of the returned string
- * @end_offset: (out): the offset of the first character after the
+ * @start_offset: the start offset of the returned string
+ * @end_offset: the offset of the first character after the 
  *              returned substring
  *
  * Gets the specified text.
  *
- * Deprecated: 2.9.3: Please use atk_text_get_string_at_offset() instead.
+ * If the boundary_type if ATK_TEXT_BOUNDARY_CHAR the character after the 
+ * offset is returned.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
+ * is from the word start after the offset to the next word start.
+ *
+ * The returned string will contain the word after the offset if the offset 
+ * is inside a word or if the offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end at or after the offset to the next work end.
+ *
+ * The returned string will contain the word after the offset if the offset
+ * is inside a word and will contain the word after the word after the offset
+ * if the offset is not inside a word.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
+ * string is from the sentence start after the offset to the next sentence
+ * start.
+ *
+ * The returned string will contain the sentence after the offset if the offset
+ * is inside a sentence or if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end at or after the offset to the next sentence end.
+ *
+ * The returned string will contain the sentence after the offset if the offset
+ * is inside a sentence and will contain the sentence after the sentence
+ * after the offset if the offset is not inside a sentence.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
+ * string is from the line start after the offset to the next line start.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end at or after the offset to the next line end.
  *
  * Returns: a newly allocated string containing the text after @offset bounded
- *          by the specified @boundary_type. Use g_free() to free the returned
- *          string.
+ *   by the specified @boundary_type. Use g_free() to free the returned string.
  **/
 gchar*
 atk_text_get_text_after_offset (AtkText          *text,
@@ -435,8 +378,8 @@ atk_text_get_text_after_offset (AtkText          *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: (out): the starting character offset of the returned string
- * @end_offset: (out): the offset of the first character after the
+ * @start_offset: the start offset of the returned string
+ * @end_offset: the offset of the first character after the 
  *              returned substring
  *
  * Gets the specified text.
@@ -445,11 +388,19 @@ atk_text_get_text_after_offset (AtkText          *text,
  * offset is returned.
  *
  * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
- * is from the word start at or before the offset to the word start after
+ * is from the word start at or before the offset to the word start after 
  * the offset.
  *
  * The returned string will contain the word at the offset if the offset
- * is inside a word and will contain the word before the offset if the
+ * is inside a word and will contain the word before the offset if the 
+ * offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end before the offset to the word end at or after the
+ * offset.
+ *
+ * The returned string will contain the word at the offset if the offset
+ * is inside a word and will contain the word after to the offset if the 
  * offset is not inside a word.
  *
  * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
@@ -457,19 +408,27 @@ atk_text_get_text_after_offset (AtkText          *text,
  * start after the offset.
  *
  * The returned string will contain the sentence at the offset if the offset
- * is inside a sentence and will contain the sentence before the offset
+ * is inside a sentence and will contain the sentence before the offset 
+ * if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end before the offset to the sentence end at or
+ * after the offset.
+ *
+ * The returned string will contain the sentence at the offset if the offset
+ * is inside a sentence and will contain the sentence after the offset 
  * if the offset is not inside a sentence.
  *
  * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
  * string is from the line start at or before the offset to the line
  * start after the offset.
  *
- * Deprecated: This method is deprecated since ATK version
- * 2.9.4. Please use atk_text_get_string_at_offset() instead.
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end before the offset to the line end at or after
+ * the offset.
  *
- * Returns: a newly allocated string containing the text at @offset bounded
- *          by the specified @boundary_type. Use g_free() to free the returned
- *          string.
+ * Returns: a newly allocated string containing the text at @offset bounded by
+ *   the specified @boundary_type. Use g_free() to free the returned string.
  **/
 gchar*
 atk_text_get_text_at_offset (AtkText          *text,
@@ -506,17 +465,55 @@ atk_text_get_text_at_offset (AtkText          *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: (out): the starting character offset of the returned string
- * @end_offset: (out): the offset of the first character after the
+ * @start_offset: the start offset of the returned string
+ * @end_offset: the offset of the first character after the 
  *              returned substring
  *
  * Gets the specified text.
  *
- * Deprecated: 2.9.3: Please use atk_text_get_string_at_offset() instead.
+ * If the boundary_type if ATK_TEXT_BOUNDARY_CHAR the character before the
+ * offset is returned.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START the returned string
+ * is from the word start before the word start before or at the offset to 
+ * the word start before or at the offset.
+ *
+ * The returned string will contain the word before the offset if the offset
+ * is inside a word and will contain the word before the word before the 
+ * offset if the offset is not inside a word.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_END the returned string
+ * is from the word end before the word end before the offset to the word
+ * end before the offset.
+ *
+ * The returned string will contain the word before the offset if the offset
+ * is inside a word or if the offset is not inside a word.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_SENTENCE_START the returned
+ * string is from the sentence start before the sentence start before 
+ * the offset to the sentence start before the offset.
+ *
+ * The returned string will contain the sentence before the offset if the 
+ * offset is inside a sentence and will contain the sentence before the 
+ * sentence before the offset if the offset is not inside a sentence.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_SENTENCE_END the returned string
+ * is from the sentence end before the sentence end at or before the offset to 
+ * the sentence end at or before the offset.
+ *
+ * The returned string will contain the sentence before the offset if the 
+ * offset is inside a sentence or if the offset is not inside a sentence.
+ *
+ * If the boundary type is ATK_TEXT_BOUNDARY_LINE_START the returned
+ * string is from the line start before the line start ar or before the offset 
+ * to the line start ar or before the offset.
+ *
+ * If the boundary_type is ATK_TEXT_BOUNDARY_LINE_END the returned string
+ * is from the line end before the line end before the offset to the 
+ * line end before the offset.
  *
  * Returns: a newly allocated string containing the text before @offset bounded
- *          by the specified @boundary_type. Use g_free() to free the returned
- *          string.
+ *   by the specified @boundary_type. Use g_free() to free the returned string.
  **/
 gchar*
 atk_text_get_text_before_offset (AtkText          *text,
@@ -551,102 +548,13 @@ atk_text_get_text_before_offset (AtkText          *text,
     return NULL;
 }
 
-/**
- * atk_text_get_string_at_offset:
- * @text: an #AtkText
- * @offset: position
- * @granularity: An #AtkTextGranularity
- * @start_offset: (out): the starting character offset of the returned string, or -1
- *                in the case of error (e.g. invalid offset, not implemented)
- * @end_offset: (out): the offset of the first character after the returned string,
- *              or -1 in the case of error (e.g. invalid offset, not implemented)
- *
- * Gets a portion of the text exposed through an #AtkText according to a given @offset
- * and a specific @granularity, along with the start and end offsets defining the
- * boundaries of such a portion of text.
- *
- * If @granularity is ATK_TEXT_GRANULARITY_CHAR the character at the
- * offset is returned.
- *
- * If @granularity is ATK_TEXT_GRANULARITY_WORD the returned string
- * is from the word start at or before the offset to the word start after
- * the offset.
- *
- * The returned string will contain the word at the offset if the offset
- * is inside a word and will contain the word before the offset if the
- * offset is not inside a word.
- *
- * If @granularity is ATK_TEXT_GRANULARITY_SENTENCE the returned string
- * is from the sentence start at or before the offset to the sentence
- * start after the offset.
- *
- * The returned string will contain the sentence at the offset if the offset
- * is inside a sentence and will contain the sentence before the offset
- * if the offset is not inside a sentence.
- *
- * If @granularity is ATK_TEXT_GRANULARITY_LINE the returned string
- * is from the line start at or before the offset to the line
- * start after the offset.
- *
- * If @granularity is ATK_TEXT_GRANULARITY_PARAGRAPH the returned string
- * is from the start of the paragraph at or before the offset to the start
- * of the following paragraph after the offset.
- *
- * Since: 2.10
- *
- * Returns: (nullable): a newly allocated string containing the text at
- *          the @offset bounded by the specified @granularity. Use g_free()
- *          to free the returned string.  Returns %NULL if the offset is invalid
- *          or no implementation is available.
- **/
-gchar* atk_text_get_string_at_offset (AtkText *text,
-                                      gint offset,
-                                      AtkTextGranularity granularity,
-                                      gint *start_offset,
-                                      gint *end_offset)
-{
-  AtkTextIface *iface;
-  gint local_start_offset, local_end_offset;
-  gint *real_start_offset, *real_end_offset;
-
-  g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
-
-  if (start_offset)
-    {
-      *start_offset = -1;
-      real_start_offset = start_offset;
-    }
-  else
-    real_start_offset = &local_start_offset;
-
-  if (end_offset)
-    {
-      *end_offset = -1;
-      real_end_offset = end_offset;
-    }
-  else
-    real_end_offset = &local_end_offset;
-
-  if (offset < 0)
-    return NULL;
-
-  iface = ATK_TEXT_GET_IFACE (text);
-
-  if (iface->get_string_at_offset)
-    return (*(iface->get_string_at_offset)) (text, offset, granularity, real_start_offset, real_end_offset);
-  else
-    return NULL;
-}
-
 /**
  * atk_text_get_caret_offset:
  * @text: an #AtkText
  *
- * Gets the offset of the position of the caret (cursor).
+ * Gets the offset position of the caret (cursor).
  *
- * Returns: the character offset of the position of the caret or -1 if
- *          the caret is not located inside the element or in the case of
- *          any other failure.
+ * Returns: the offset position of the caret (cursor).
  **/
 gint
 atk_text_get_caret_offset (AtkText *text)
@@ -660,22 +568,19 @@ atk_text_get_caret_offset (AtkText *text)
   if (iface->get_caret_offset)
     return (*(iface->get_caret_offset)) (text);
   else
-    return -1;
+    return 0;
 }
 
 /**
  * atk_text_get_character_extents:
  * @text: an #AtkText
  * @offset: The offset of the text character for which bounding information is required.
- * @x: (out) (optional): Pointer for the x coordinate of the bounding box
- * @y: (out) (optional): Pointer for the y coordinate of the bounding box
- * @width: (out) (optional): Pointer for the width of the bounding box
- * @height: (out) (optional): Pointer for the height of the bounding box
+ * @x: Pointer for the x cordinate of the bounding box
+ * @y: Pointer for the y cordinate of the bounding box
+ * @width: Pointer for the width of the bounding box
+ * @height: Pointer for the height of the bounding box
  * @coords: specify whether coordinates are relative to the screen or widget window 
  *
- * If the extent can not be obtained (e.g. missing support), all of x, y, width,
- * height are set to -1.
- *
  * Get the bounding box containing the glyph representing the character at 
  *     a particular text offset. 
  **/
@@ -686,7 +591,7 @@ atk_text_get_character_extents (AtkText *text,
                                 gint *y,
                                 gint *width,
                                 gint *height,
-                                AtkCoordType coords)
+			        AtkCoordType coords)
 {
   AtkTextIface *iface;
   gint local_x, local_y, local_width, local_height;
@@ -711,10 +616,10 @@ atk_text_get_character_extents (AtkText *text,
   else
     real_height = &local_height;
 
-  *real_x = -1;
-  *real_y = -1;
-  *real_width = -1;
-  *real_height = -1;
+  *real_x = 0;
+  *real_y = 0;
+  *real_width = 0;
+  *real_height = 0;
 
   if (offset < 0)
     return;
@@ -734,10 +639,10 @@ atk_text_get_character_extents (AtkText *text,
 /**
  * atk_text_get_run_attributes:
  *@text: an #AtkText
- *@offset: the character offset at which to get the attributes, -1 means the offset of
+ *@offset: the offset at which to get the attributes, -1 means the offset of
  *the character to be inserted at the caret location.
- *@start_offset: (out): the address to put the start offset of the range
- *@end_offset: (out): the address to put the end offset of the range
+ *@start_offset: the address to put the start offset of the range
+ *@end_offset: the address to put the end offset of the range
  *
  *Creates an #AtkAttributeSet which consists of the attributes explicitly
  *set at the position @offset in the text. @start_offset and @end_offset are
@@ -748,8 +653,8 @@ atk_text_get_character_extents (AtkText *text,
  *returned.
  *
  *Returns: (transfer full): an #AtkAttributeSet which contains the attributes
- *         explicitly set at @offset. This #AtkAttributeSet should be freed by
- *         a call to atk_attribute_set_free().
+ * explicitly set at @offset. This #AtkAttributeSet should be freed by a call
+ * to atk_attribute_set_free().
  **/
 AtkAttributeSet* 
 atk_text_get_run_attributes (AtkText          *text,
@@ -792,11 +697,11 @@ atk_text_get_run_attributes (AtkText          *text,
  *attributes that can be returned. Note that other attributes may also be 
  *returned.
  *
- *Returns: (transfer full): an #AtkAttributeSet which contains the default text
- *          attributes for this #AtkText. This #AtkAttributeSet should be freed by
- *          a call to atk_attribute_set_free().
+ *Returns: (transfer full): an #AtkAttributeSet which contains the default
+ * values of attributes.  at @offset. this #atkattributeset should be freed by
+ * a call to atk_attribute_set_free().
  */
-AtkAttributeSet*
+AtkAttributeSet* 
 atk_text_get_default_attributes (AtkText          *text)
 {
   AtkTextIface *iface;
@@ -817,7 +722,7 @@ atk_text_get_default_attributes (AtkText          *text)
  *
  * Gets the character count.
  *
- * Returns: the number of characters or -1 in case of failure.
+ * Returns: the number of characters.
  **/
 gint
 atk_text_get_character_count (AtkText *text)
@@ -846,8 +751,8 @@ atk_text_get_character_count (AtkText *text)
  * are interpreted as being relative to the screen or this widget's window
  * depending on @coords.
  *
- * Returns: the offset to the character which is located at  the specified
- *          @x and @y coordinates of -1 in case of failure.
+ * Returns: the offset to the character which is located at
+ * the specified @x and @y coordinates.
  **/
 gint
 atk_text_get_offset_at_point (AtkText *text,
@@ -873,7 +778,8 @@ atk_text_get_offset_at_point (AtkText *text,
  *
  * Gets the number of selected regions.
  *
- * Returns: The number of selected regions, or -1 in the case of failure.
+ * Returns: The number of selected regions, or -1 if a failure
+ *   occurred.
  **/
 gint
 atk_text_get_n_selections (AtkText *text)
@@ -898,14 +804,14 @@ atk_text_get_n_selections (AtkText *text)
  * start of the text.  The selected region closest to the beginning
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
- * @start_offset: (out): passes back the starting character offset of the selected region
- * @end_offset: (out): passes back the ending character offset (offset immediately past)
- * of the selected region
+ * @start_offset: passes back the start position of the selected region
+ * @end_offset: passes back the end position of (e.g. offset immediately past) 
+ * the selected region
  *
  * Gets the text from the specified selection.
  *
  * Returns: a newly allocated string containing the selected text. Use g_free()
- *          to free the returned string.
+ *   to free the returned string.
  **/
 gchar*
 atk_text_get_selection (AtkText *text, 
@@ -942,12 +848,12 @@ atk_text_get_selection (AtkText *text,
 /**
  * atk_text_add_selection:
  * @text: an #AtkText
- * @start_offset: the starting character offset of the selected region
+ * @start_offset: the start position of the selected region
  * @end_offset: the offset of the first character after the selected region.
  *
  * Adds a selection bounded by the specified offsets.
  *
- * Returns: %TRUE if successful, %FALSE otherwise
+ * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
 atk_text_add_selection (AtkText *text, 
@@ -977,7 +883,7 @@ atk_text_add_selection (AtkText *text,
  *
  * Removes the specified selection.
  *
- * Returns: %TRUE if successful, %FALSE otherwise
+ * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
 atk_text_remove_selection (AtkText *text, 
@@ -1003,13 +909,13 @@ atk_text_remove_selection (AtkText *text,
  * start of the text.  The selected region closest to the beginning
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
- * @start_offset: the new starting character offset of the selection
+ * @start_offset: the new start position of the selection
  * @end_offset: the new end position of (e.g. offset immediately past) 
  * the selection
  *
  * Changes the start and end offset of the specified selection.
  *
- * Returns: %TRUE if successful, %FALSE otherwise
+ * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
 atk_text_set_selection (AtkText *text, 
@@ -1035,27 +941,11 @@ atk_text_set_selection (AtkText *text,
 /**
  * atk_text_set_caret_offset:
  * @text: an #AtkText
- * @offset: the character offset of the new caret position
+ * @offset: position
  *
  * Sets the caret (cursor) position to the specified @offset.
  *
- * In the case of rich-text content, this method should either grab focus
- * or move the sequential focus navigation starting point (if the application
- * supports this concept) as if the user had clicked on the new caret position.
- * Typically, this means that the target of this operation is the node containing
- * the new caret position or one of its ancestors. In other words, after this
- * method is called, if the user advances focus, it should move to the first
- * focusable node following the new caret position.
- *
- * Calling this method should also scroll the application viewport in a way
- * that matches the behavior of the application's typical caret motion or tab
- * navigation as closely as possible. This also means that if the application's
- * caret motion or focus navigation does not trigger a scroll operation, this
- * method should not trigger one either. If the application does not have a caret
- * motion or focus navigation operation, this method should try to scroll the new
- * caret position into view while minimizing unnecessary scroll motion.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
+ * Returns: %TRUE if success, %FALSE otherwise.
  **/
 gboolean
 atk_text_set_caret_offset (AtkText *text,
@@ -1085,13 +975,10 @@ atk_text_set_caret_offset (AtkText *text,
  * @end_offset: The offset of the text character after the last character 
  *        for which boundary information is required.
  * @coord_type: Specify whether coordinates are relative to the screen or widget window.
- * @rect: (out): A pointer to a AtkTextRectangle which is filled in by this function.
+ * @rect: A pointer to a AtkTextRectangle which is filled in by this function.
  *
  * Get the bounding box for text within the specified range.
  *
- * If the extents can not be obtained (e.g. or missing support), the rectangle
- * fields are set to -1.
- *
  * Since: 1.3
  **/
 void
@@ -1105,23 +992,18 @@ atk_text_get_range_extents (AtkText          *text,
 
   g_return_if_fail (ATK_IS_TEXT (text));
   g_return_if_fail (rect);
-  g_return_if_fail (start_offset >= 0 && start_offset < end_offset);
 
+  if (start_offset < 0 || start_offset >= end_offset)
+    return;
+ 
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_range_extents)
     (*(iface->get_range_extents)) (text, start_offset, end_offset, coord_type, rect);
-  else
-    {
-      rect->x = -1;
-      rect->y = -1;
-      rect->width = -1;
-      rect->height = -1;
-    }
 }
 
 /**
- * atk_text_get_bounded_ranges: (virtual get_bounded_ranges)
+ * atk_text_get_bounded_ranges:
  * @text: an #AtkText
  * @rect: An AtkTextRectangle giving the dimensions of the bounding box.
  * @coord_type: Specify whether coordinates are relative to the screen or widget window.
@@ -1251,8 +1133,8 @@ atk_text_attribute_get_name (AtkTextAttribute attr)
  * Get the #AtkTextAttribute type corresponding to a text attribute name.
  *
  * Returns: the #AtkTextAttribute enumerated type corresponding to the specified
- *          name, or #ATK_TEXT_ATTRIBUTE_INVALID if no matching text attribute
- *          is found.
+name,
+ *          or #ATK_TEXT_ATTRIBUTE_INVALID if no matching text attribute is found.
  **/
 AtkTextAttribute
 atk_text_attribute_for_name (const gchar *name)
@@ -1304,9 +1186,8 @@ atk_text_attribute_for_name (const gchar *name)
  *
  * Gets the value for the index of the #AtkTextAttribute
  *
- * Returns: (nullable): a string containing the value; this string
- * should not be freed; %NULL is returned if there are no values
- * maintained for the attr value.
+ * Returns: a string containing the value; this string should not be freed;
+ * NULL is returned if there are no values maintained for the attr value. 
  **/
 const gchar*
 atk_text_attribute_get_value (AtkTextAttribute attr,
@@ -1343,9 +1224,6 @@ atk_text_attribute_get_value (AtkTextAttribute attr,
     case ATK_TEXT_ATTR_STYLE:
       g_assert (index >= 0 && index < G_N_ELEMENTS (style_offsets));
       return style + style_offsets[index];
-    case ATK_TEXT_ATTR_TEXT_POSITION:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (text_position_offsets));
-      return text_position + text_position_offsets[index];
     default:
       return NULL;
    }
@@ -1358,23 +1236,6 @@ atk_text_rectangle_union (AtkTextRectangle *src1,
 {
   gint dest_x, dest_y;
 
-  /*
-   * Some invocations of e.g. atk_text_get_character_extents
-   * may return "-1" rectangles for character positions without support for
-   * getting an extent. In that case we have to ignore them instead of using -1
-   * values in computations.
-   */
-  if (src1->width == -1)
-    {
-      *dest = *src2;
-      return;
-    }
-  if (src2->width == -1)
-    {
-      *dest = *src1;
-      return;
-    }
-
   dest_x = MIN (src1->x, src2->x);
   dest_y = MIN (src1->y, src2->y);
   dest->width = MAX (src1->x + src1->width, src2->x + src2->width) - dest_x;
@@ -1415,72 +1276,6 @@ atk_text_rectangle_contain (AtkTextRectangle *clip,
   
 }
 
-/**
- * atk_text_scroll_substring_to:
- * @text: an #AtkText
- * @start_offset: start offset in the @text
- * @end_offset: end offset in the @text, or -1 for the end of the text.
- * @type: specify where the object should be made visible.
- *
- * Makes a substring of @text visible on the screen by scrolling all necessary parents.
- *
- * Since: 2.32
- *
- * Returns: whether scrolling was successful.
- */
-gboolean
-atk_text_scroll_substring_to (AtkText       *text,
-                              gint          start_offset,
-                              gint          end_offset,
-                              AtkScrollType type)
-{
-  AtkTextIface *iface = NULL;
-  g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
-
-  iface = ATK_TEXT_GET_IFACE (text);
-
-  if (iface->scroll_substring_to)
-    return (iface->scroll_substring_to) (text, start_offset, end_offset, type);
-  else
-    return FALSE;
-}
-
-/**
- * atk_text_scroll_substring_to_point:
- * @text: an #AtkText
- * @start_offset: start offset in the @text
- * @end_offset: end offset in the @text, or -1 for the end of the text.
- * @coords: specify whether coordinates are relative to the screen or to the
- * parent object.
- * @x: x-position where to scroll to
- * @y: y-position where to scroll to
- *
- * Move the top-left of a substring of @text to a given position of the screen
- * by scrolling all necessary parents.
- *
- * Since: 2.32
- *
- * Returns: whether scrolling was successful.
- */
-gboolean
-atk_text_scroll_substring_to_point (AtkText      *text,
-                                    gint         start_offset,
-                                    gint         end_offset,
-                                    AtkCoordType coords,
-                                    gint         x,
-                                    gint         y)
-{
-  AtkTextIface *iface = NULL;
-  g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
-
-  iface = ATK_TEXT_GET_IFACE (text);
-
-  if (iface->scroll_substring_to_point)
-    return (iface->scroll_substring_to_point) (text, start_offset, end_offset, coords, x, y);
-  else
-    return FALSE;
-}
-
 static void 
 atk_text_real_get_range_extents (AtkText           *text,
                                  gint              start_offset,
@@ -1589,8 +1384,7 @@ atk_text_real_get_bounded_ranges (AtkText          *text,
 
 /**
  * atk_text_free_ranges:
- * @ranges: (array): A pointer to an array of #AtkTextRange which is
- *   to be freed.
+ * @ranges: A pointer to an array of  #AtkTextRange which is to be freed.
  *
  * Frees the memory associated with an array of AtkTextRange. It is assumed
  * that the array was returned by the function atk_text_get_bounded_ranges
